/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

/*
 * this is the code that a user would typically modify
 *   for their own purposes.
 *
 * The following routines must be provided.
 *
 * user_event()  : executed for each event
 *     arguments :
 *         int *buffer;        pointer to buffer area
 *         int *length;        available buffer length
 *         int *event_length;  user_event must set the event length
 *         int *event_ID;      user_event must set the "event ID" word
 *         int *max_length;    max length (words) an event buffer may occupy
 *         int *mode;          mode of event taking, for those using it
 *     return    :
 *         ERANGE              buffer length is not enough for the event.
 *         ETIMEDOUT           timed out on waiting an event
 *         0                   normal return   
 *
 * user_init()   : executed once at the beginning 
 *     arguments :
 *         int *max_length;    maximum event length in words.
 * user_begin()  : executed on "begin" command
 *     arguments :
 *         int *runtype;       runtype variable, for those using it.
 *         int *max_length;    maximum event length in words.
 * user_resume() : executed on "resume" command
 *     arguments :
 *         int *mode;          mode of event taking, for those using it
 * user_pause()  : executed on "pause" command
 *     arguments :
 *         int *mode;          mode of event taking, for those using it
 * user_end()    : executed on "end" command
 *     arguments :
 *         int *runtype;       runtype variable, for those using it.
 * user_exit()   : executed once at the end
 *
 */

#include "template.h"
extern int debug_level;

#if !defined(SGI) && !defined(VXWORKS)
#include "camlib.h"
#endif

#ifdef VXWORKS
#define ERANGE     34   /* ERANGE and ETIMEDOUT hard-coded from HP errno.h */
#define ETIMEDOUT 238
#else
#include <errno.h>
#endif

/* define CAMAC station Number */
#define INTREG 5

/* Maximum event size */
#define MAX_EVENT_SIZE 100

/* Handles for event source */
extern char eventsource[20];
int eventSrcType;
#define PSEUDO 1
#define OTHER  0

user_init( max_length ) int *max_length; {
  if (strcmp(eventsource,"PSEUDO") == 0) {
    eventSrcType = PSEUDO;
    *max_length = MAX_EVENT_SIZE;
  }
  else  {
    *max_length = MAX_EVENT_SIZE;
    eventSrcType = OTHER;
  }
  return;
}

user_event(buffer,length,event_length,event_ID,max_length,mode)
     int *buffer;       /* pointer to buffer                            */
     int *length;       /* buffer length                                */
     int *event_length; /* user_event must set the event length         */
     int *event_ID;     /* user_event must set the event ID word        */
     int *max_length;   /* max words an event should occupy             */
     int *mode;         /* mode of event taking, for those who use it   */
{

  int dat,i,q,x;
  int mask, status;

  /*
   * Check the buffer size
   * If the given length is not enough to write one event,
   * error code "ERANGE" must be returned.
   */
  if (*length < *max_length) return ERANGE;
  
  *event_length=0;

  if (eventSrcType == PSEUDO)  {                         /* PSEUDO event */
    pseudo_event ( max_length, event_length, buffer );
    *event_ID = EVENT_RECORD;
    return 0;
  }
  else   {                                               /* OTHER event */

#if !defined(SGI) && !defined(VXWORKS)
  /*
   * Wait event.
   * On timed out, control must be returned to main routine.
   */
    status = CAM_WaitLAM(3);
    CAMAC(NAF(INTREG,0,9),&dat,&q,&x);     /* Clear LAM */
    if (status==ETIMEDOUT) {               /* timedout */
      if (debug_level >= 10) printf("LAM wait timedout\n");
      return status;
    }
    else if (status==EINTR) {              /* signal */
      if (debug_level >= 10) printf("LAM wait interrupted\n");
      return status;
    }
    if (status) {                          /* Error */
      if (debug_level >= 10) printf("LAM wait unknown status %d\n",status);
      return status;
    }
  
  /*
   * flush_buffer("","") force to forward the nova buffer to the
   * processes on the down-stream.
   * event_length and event_ID must be ready on calling "flush_buffer()"
   */
    if (*mode == 0)  {
      *event_ID = EVENT_RECORD;
      *event_length=10;
    }
    else  {
      *event_ID = ENVIRONMENT_RECORD;
      *event_length=100;
      flush_buffer("","");
    }

  /*
   * end of one event 
   */

#endif
    return 0;
  }                                /*  End of OTHER event source switch  */
}

user_begin(run_type,max_length) int *run_type,*max_length; {

  if (strcmp(eventsource,"PSEUDO") == 0)  eventSrcType = PSEUDO;
  else                                    eventSrcType = OTHER;

  if (eventSrcType == PSEUDO)  {
    *max_length = 150;
    pseudo_begin();
    return 0;
  }
  else {
#if !defined(SGI) && !defined(VXWORKS)
    CAMOPN();
    CSETBR(1);
    CSETCR(0);
    CGENC();
    CGENZ();
    CREMI();
    CAM_DisableLAM();
#endif
    return;
  }
}

user_pause( mode ) int *mode; {
  int mask; int dat,q,x;
  
  if (eventSrcType == PSEUDO)  {
    pseudo_pause();
    return 0;
  }
  else {
#if !defined(SGI) && !defined(VXWORKS)
    CAMAC(NAF(INTREG,0,24),&dat,&q,&x);   /* Disable LAM */
    CAM_DisableLAM();
#endif
    return;
  }
}

user_resume( mode ) int *mode; {   
  int mask; int dat,q,x;

  if (eventSrcType == PSEUDO)  {
    pseudo_resume();
    return 0;
  }
  else {
#if !defined(SGI) && !defined(VXWORKS)
    mask = 1 << (INTREG - 1);
    CAM_EnableLAM(mask);
    CAMAC(NAF(INTREG,0,26),&dat,&q,&x);   /* Enable LAM */
#endif
    return;
  }
}

user_end( run_type ) int *run_type; {
  if (eventSrcType == PSEUDO)  {
    pseudo_end();
    return 0;
  }
  else {
#if !defined(SGI) && !defined(VXWORKS)
    CAM_DisableLAM();
    CAM_Close();
#endif
    return;
  }
}

user_exit() {
   return;
}



